@@ -15,6 +15,7 @@ TrackView::TrackView(Spotify::Session* pSession, QWidget* parent) :
 
         connect(m_pQueueView,SIGNAL(playQueuePosUpdated(qint32)),SLOT(OnPlayQueuePosUpdated(qint32)));
         connect(&m_TrackInfo,SIGNAL(itemActivated(QTableWidgetItem*)),SLOT(OnTrackActivated(QTableWidgetItem*)));
+        connect(m_TrackInfo.horizontalHeader(),SIGNAL(sectionClicked(int)),SLOT(OnHeaderClicked(int)));
 
         //set table view properties
         m_TrackInfo.setColumnCount(3);
@@ -22,6 +23,13 @@ TrackView::TrackView(Spotify::Session* pSession, QWidget* parent) :
         m_TrackInfo.setColumnWidth(1,200);
         m_TrackInfo.setColumnWidth(2,200);
         m_TrackInfo.setSelectionMode(QAbstractItemView::SingleSelection);
+        m_TrackInfo.setShowGrid(false); //no grid should be displayed
+        m_TrackInfo.verticalHeader()->hide(); //hide default vertical header
+        m_TrackInfo.horizontalScrollBar()->hide();
+        m_TrackInfo.horizontalHeader()->setStretchLastSection(true);
+        m_TrackInfo.horizontalHeader()->setResizeMode(0, QHeaderView::Stretch); //  column 0
+        m_TrackInfo.horizontalHeader()->setClickable(true);
+
 
         m_Layout.addWidget(&m_TrackInfo);
 
@@ -45,6 +53,9 @@ void TrackView::setPlaylist(Spotify::Playlist* pPlaylist)
         {
             m_pPlaylist = pPlaylist;
 
+            m_currSortOrder = SORT_ORIGINAL;
+            m_currSortCol = -1;
+
             setWindowTitle(pPlaylist->getName());
 
             addTracks(m_pPlaylist); //add track data to track list
@@ -61,7 +72,12 @@ void TrackView::setSearchResults(Spotify::Search* pSearch)
     {
         setWindowTitle(tr("Search results: ") + pSearch->getSearchQuery());
 
+        m_currSortOrder = SORT_ORIGINAL;
+        m_currSortCol = -1;
+
         m_TrackList = pSearch->getTracks();
+	//sort track instance first by artist, then by album, and finally by track
+	qSort(m_TrackList.begin(),m_TrackList.end(),trackLessThanCaseInsensitive); 
         addTracks(m_TrackList);
     }
 }
@@ -144,6 +160,111 @@ void TrackView::OnPlayQueuePosUpdated(qint32 pos)
     emit playQueuePosUpdated(pos);
 }
 
+void TrackView::OnHeaderClicked(int logicalIndex)
+{
+    //A header has been clicked, perform sorting of tracks
+    //based on selected column and sort order
+
+    if(logicalIndex!=m_currSortCol)
+    {
+        m_currSortOrder = SORT_ASCENDING;
+        m_currSortCol = logicalIndex;
+        if(m_pPlaylist) //playlist mode
+        {
+            if(m_currSortCol == COLUMN_TRACK)
+                m_pPlaylist->sortByTracks(true);
+            else if(m_currSortCol == COLUMN_ARTIST)
+                m_pPlaylist->sortByArtists(true);
+            else if(m_currSortCol == COLUMN_ALBUM)
+                m_pPlaylist->sortByAlbums(true);
+            addTracks(m_pPlaylist); //refresh playlist view
+        }
+        else
+        {
+            //search mode
+            if(m_currSortCol == COLUMN_TRACK)
+                qSort(m_TrackList.begin(),m_TrackList.end(),tracksByNameAsc);
+            else if(m_currSortCol == COLUMN_ARTIST)
+                qSort(m_TrackList.begin(),m_TrackList.end(),tracksByArtistNameAsc);
+            else if(m_currSortCol == COLUMN_ALBUM)
+                qSort(m_TrackList.begin(),m_TrackList.end(),tracksByAlbumNameAsc);
+            addTracks(m_TrackList); //refresh track view
+        }
+    }
+    else
+    { //same column clicked, change the sort order
+        if(m_currSortOrder==SORT_ORIGINAL)
+            m_currSortOrder=SORT_ASCENDING;
+        else if(m_currSortOrder==SORT_ASCENDING)
+            m_currSortOrder=SORT_DECENDING;
+        else if(m_currSortOrder==SORT_DECENDING)
+            m_currSortOrder=SORT_ORIGINAL;
+        if(m_pPlaylist) //playlist mode
+        {
+            if(m_currSortOrder==SORT_ORIGINAL)
+            {
+                m_pPlaylist->restoreOrignalCollection();
+            }
+            else
+            {
+                if(m_currSortCol==COLUMN_TRACK)
+                {
+                    if(m_currSortOrder==SORT_ASCENDING)
+                        m_pPlaylist->sortByTracks(true);
+                    else
+                        m_pPlaylist->sortByTracks(false);
+                }
+                else if(m_currSortCol==COLUMN_ARTIST)
+                {
+                    if(m_currSortOrder==SORT_ASCENDING)
+                        m_pPlaylist->sortByArtists(true);
+                    else
+                        m_pPlaylist->sortByArtists(false);
+                }
+                else if(m_currSortCol==COLUMN_ALBUM)
+                {
+                    if(m_currSortOrder==SORT_ASCENDING)
+                        m_pPlaylist->sortByAlbums(true);
+                    else
+                        m_pPlaylist->sortByAlbums(false);
+                }
+            }
+            addTracks(m_pPlaylist);
+        }
+        else
+        { //search mode
+            if(m_currSortOrder==SORT_ORIGINAL)
+            {
+                qSort(m_TrackList.begin(),m_TrackList.end(),trackLessThanCaseInsensitive);
+            }
+            else
+            {
+                if(m_currSortCol==COLUMN_TRACK)
+                {
+                    if(m_currSortOrder==SORT_ASCENDING)
+                       qSort(m_TrackList.begin(),m_TrackList.end(),tracksByNameAsc);
+                    else
+                        qSort(m_TrackList.begin(),m_TrackList.end(),tracksByNameDesc);
+                }
+                else if(m_currSortCol==COLUMN_ARTIST)
+                {
+                    if(m_currSortOrder==SORT_ASCENDING)
+                        qSort(m_TrackList.begin(),m_TrackList.end(),tracksByArtistNameAsc);
+                    else
+                        qSort(m_TrackList.begin(),m_TrackList.end(),tracksByArtistNameDesc);
+                }
+                else if(m_currSortCol==COLUMN_ALBUM)
+                {
+                    if(m_currSortOrder==SORT_ASCENDING)
+                       qSort(m_TrackList.begin(),m_TrackList.end(),tracksByAlbumNameAsc);
+                    else
+                        qSort(m_TrackList.begin(),m_TrackList.end(),tracksByAlbumNameDesc);
+                }
+            }
+            addTracks(m_TrackList);
+        }
+    }
+}
 
 void TrackView::addTracks(Spotify::Playlist* playlist)
 {
@@ -151,7 +272,7 @@ void TrackView::addTracks(Spotify::Playlist* playlist)
 
     QStringList headers;
 
-    headers << "Track" << "Artist" << "Album";
+    headers << tr("Track") << tr("Artist") << tr("Album");
 
     setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display progress indicator
 
@@ -177,16 +298,20 @@ void TrackView::addTracks(Spotify::Playlist* playlist)
                                 QTableWidgetItem* albumName = new QTableWidgetItem();
 
                                 trackName->setText(currTrack->getName());
+                                trackName->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
                                 trackName->setData(Qt::UserRole,trackPos); //associate position in track instances list with list item
                                 if(currTrack->getArtist())
                                     artistName->setText(currTrack->getArtist()->getName());
+                                artistName->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
                                 artistName->setData(Qt::UserRole,trackPos);
                                 if(currTrack->getAlbum())
                                     albumName->setText(currTrack->getAlbum()->getName());
+                                albumName->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
                                 albumName->setData(Qt::UserRole,trackPos);
                                 m_TrackInfo.setItem(row,0,trackName);
                                 m_TrackInfo.setItem(row,1,artistName);
                                 m_TrackInfo.setItem(row,2,albumName);
+                                m_TrackInfo.setRowHeight(row,70); //set default row size to 70px
                                 row++;
                             }
                     }
@@ -239,6 +364,7 @@ void TrackView::addTracks(QList<Spotify::Track*> trackList)
                  m_TrackInfo.setItem(row,0,trackName);
                  m_TrackInfo.setItem(row,1,artistName);
                  m_TrackInfo.setItem(row,2,albumName);
+                 m_TrackInfo.setRowHeight(row,70); //set default row height to 70px
                  row++;
              }
      }
