@@ -354,10 +354,11 @@ Spotify::Error Spotify::Session::getAlbumCover( Spotify::Album* album )
     return error;
 }
 
-Spotify::Error Spotify::Session::getAlbumCover( Spotify::Track* track )
+/*Spotify::Error Spotify::Session::getAlbumCover( Spotify::Track* track )
 {
     return getAlbumCover( track->getAlbum() );
 }
+*/
 
 void Spotify::Session::getPlaylistContainer()
 {
@@ -1002,7 +1003,7 @@ void Spotify::Session::OnImageLoaded(sp_image* image, void* userdata)
     // Release sp_image:
     sp_image_release(image);
 
-    if(userdata)
+    /*if(userdata)
     {
         qDebug() << "Assigning image container";
         Spotify::ImageContainer* ic = (Spotify::ImageContainer*)userdata;
@@ -1011,6 +1012,7 @@ void Spotify::Session::OnImageLoaded(sp_image* image, void* userdata)
             ic->setImage(img); 
         }
     }
+    */
 
     emit coverArtReady(img);
 }
